package org.yarrahouse;

import java.awt.Cursor;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SpringLayout;

import net.iharder.dnd.FileDrop;

import java.awt.Font;
import java.awt.Color;

public class FileConcatenator
{

    private JFrame frame;
    private JProgressBar progressBar;
    private final Action actionSaveAs = new SaveAs();
    private final Action actionExit   = new Exit();
    private final Action actionAbout  = new About();
    List<File>           fileList     = new ArrayList<File>();

    /**
     * Launch the application.
     */
    public static void main(String[] args)
    {
        EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                try
                {
                    FileConcatenator window = new FileConcatenator();
                    window.frame.setVisible(true);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * Create the application.
     */
    public FileConcatenator()
    {
        initialize();
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize()
    {
        frame = new JFrame(Messages.getString("FileConcatenator.title")); //$NON-NLS-1$
        frame.setForeground(Color.BLUE);
        frame.setBackground(Color.WHITE);
        frame.setBounds(100, 100, 450, 300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        JMenuBar menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);
        
        JMenu mnFile = new JMenu(Messages.getString("FileConcatenator.file")); //$NON-NLS-1$
        menuBar.add(mnFile);
        
        final JMenuItem mntmSaveAs = new JMenuItem(Messages.getString("FileConcatenator.saveAs")); //$NON-NLS-1$
        mntmSaveAs.setAction(actionSaveAs);
        mnFile.add(mntmSaveAs);

        JMenuItem mntmExit = new JMenuItem(Messages.getString("FileConcatenator.exit")); //$NON-NLS-1$
        mntmExit.setAction(actionExit);
        mnFile.add(mntmExit);
        
        JMenu mnHelp = new JMenu(Messages.getString("FileConcatenator.help")); //$NON-NLS-1$
        menuBar.add(mnHelp);
        
        JMenuItem mntmAbout = new JMenuItem(Messages.getString("FileConcatenator.about")); //$NON-NLS-1$
        mntmAbout.setAction(actionAbout);
        mnHelp.add(mntmAbout);
        SpringLayout springLayout = new SpringLayout();
        frame.getContentPane().setLayout(springLayout);
        
        progressBar = new JProgressBar(0,100);
        progressBar.setStringPainted(true);
        springLayout.putConstraint(SpringLayout.NORTH, progressBar, 10, SpringLayout.NORTH, frame.getContentPane());
        springLayout.putConstraint(SpringLayout.WEST, progressBar, 10, SpringLayout.WEST, frame.getContentPane());
        springLayout.putConstraint(SpringLayout.SOUTH, progressBar, 26, SpringLayout.NORTH, frame.getContentPane());
        springLayout.putConstraint(SpringLayout.EAST, progressBar, -10, SpringLayout.EAST, frame.getContentPane());
        frame.getContentPane().add(progressBar);
        
        JScrollPane scrollPane = new JScrollPane();
        springLayout.putConstraint(SpringLayout.NORTH, scrollPane, 6, SpringLayout.SOUTH, progressBar);
        springLayout.putConstraint(SpringLayout.WEST, scrollPane, 10, SpringLayout.WEST, frame.getContentPane());
        springLayout.putConstraint(SpringLayout.SOUTH, scrollPane, -10, SpringLayout.SOUTH, frame.getContentPane());
        springLayout.putConstraint(SpringLayout.EAST, scrollPane, 0, SpringLayout.EAST, progressBar);
        frame.getContentPane().add(scrollPane);
        
        final JTextArea fileListTextArea = new JTextArea(Messages.getString("FileConcatenator.dropFilesHere")); //$NON-NLS-1$
        fileListTextArea.setToolTipText(Messages.getString("FileConcatenator.toolTip")); //$NON-NLS-1$
        fileListTextArea.setLineWrap(true);
        fileListTextArea.setForeground(Color.BLACK);
        fileListTextArea.setFont(new Font("Courier New", Font.PLAIN, 13));
        fileListTextArea.setEditable(false);
        scrollPane.setViewportView(fileListTextArea);
        mntmSaveAs.setEnabled(false);
        frame.setLocationRelativeTo(null);
        new FileDrop( /*System.out,*/ fileListTextArea, /*dragBorder,*/ new FileDrop.Listener()
        {
            public void filesDropped(File[] files)
            {
                fileListTextArea.setText(null); // Clear out the Text Area
                fileList = new ArrayList<File>(); // clear out the existing list
                for( int i = 0; i < files.length; i++ )
                {
                    try
                    {
                        if (files[i].isFile()) // Copy only files
                        {
                            fileListTextArea.append( files[i].getCanonicalPath() + "\n" ); //$NON-NLS-1$
                            fileList.add(files[i]); // Create a list of files to concatenate
                        }
                        // If there are files to merge enable SaveAs
                        // must be at least two files
                        mntmSaveAs.setEnabled(fileList.size() > 1); 
                    }
                    // end try
                    catch( java.io.IOException e ) 
                    {
                    }
                }   // end for: through each dropped file
            }   // end filesDropped
        }); // end FileDrop.Listener
    }

    private class SaveAs extends AbstractAction 
    {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
        
        public SaveAs() 
        {
            putValue(NAME, Messages.getString("FileConcatenator.saveAs")); //$NON-NLS-1$
            putValue(SHORT_DESCRIPTION, Messages.getString("FileConcatenator.saveAsDescription")); //$NON-NLS-1$
        }
        public void actionPerformed(ActionEvent e) 
        {
            // Enable only when there are at least two files
            assert !fileList.isEmpty(); 
            try
            {
            JFileChooser     fileChooser; 
            String           directory; //$NON-NLS-1$
            int              iVal;
            
                directory = fileList.get(0).getCanonicalPath();
                fileChooser = new JFileChooser(directory);
                iVal = fileChooser.showSaveDialog(frame.getParent());
                if (iVal == JFileChooser.APPROVE_OPTION) 
                {
                    // Kick off the worker thread to merge the files
                    FilesMergeTask task = new FilesMergeTask(fileList, fileChooser.getSelectedFile());
                    task.addPropertyChangeListener(new PropertyChangeListener() {
                            public  void propertyChange(PropertyChangeEvent evt) 
                            {
                                if ("progress".equals(evt.getPropertyName()))  //$NON-NLS-1$
                                {
                                    progressBar.setValue((Integer)evt.getNewValue());
                                }
                            }
                        });
                    task.execute();
                }      
            }
            catch (IOException e1)
            {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }   
        }
    }

    private class Exit extends AbstractAction 
    {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
        public Exit() 
        {
            putValue(NAME, Messages.getString("FileConcatenator.exit")); //$NON-NLS-1$
            putValue(SHORT_DESCRIPTION, Messages.getString("FileConcatenator.exitDescription")); //$NON-NLS-1$
        }
        public void actionPerformed(ActionEvent e) 
        {
            frame.setVisible(false);
            frame.dispose();
        }
    }
    private class About extends AbstractAction 
    {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    AboutDialog dialog = new AboutDialog();
    
        public About() 
        {
            putValue(NAME, Messages.getString("FileConcatenator.about")); //$NON-NLS-1$
            putValue(SHORT_DESCRIPTION, Messages.getString("FileConcatenator.aboutDescription")); //$NON-NLS-1$
            dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        }
        public void actionPerformed(ActionEvent e) 
        {
            dialog.setVisible(true);
        }
    }
}
